from .fixed_point_tokenomics import FixedPointTokenomics
from .floating_point_tokenomics import FloatingPointTokenomics
from .tokenomics_base import TokenomicsBase
from .ledgers import (
    VaultLedger,
    StakingLedger,
    ExternalProtectionWalletLedger,
    PoolTokenSupplyLedger
)


# from mesa import Agent
#
#
# class RandomWalker(Agent):
#     """
#     Class implementing random walker methods in a generalized manner.
#     Not intended to be used on its own, but to inherit its methods to multiple
#     other agents.
#     """
#
#     grid = None
#     x = None
#     y = None
#     # use a Moore neighborhood
#     moore = True
#
#     def __init__(self, unique_id, pos, model, moore=True):
#         """
#         grid: The MultiGrid object in which the agent lives.
#         x: The agent's current x coordinate
#         y: The agent's current y coordinate
#         moore: If True, may move in all 8 directions.
#                 Otherwise, only up, down, left, right.
#         """
#         super().__init__(unique_id, model)
#         self.pos = pos
#         self.moore = moore
#
#     def random_move(self):
#         """
#         Step one cell in any allowable direction.
#         """
#         # Pick the next cell from the adjacent cells.
#         next_moves = self.model.grid.get_neighborhood(self.pos, self.moore, True)
#         next_move = self.random.choice(next_moves)
#         # Now move:
#         self.model.grid.move_agent(self, next_move)



"""Base class for all ledgers."""
from abc import ABC, abstractmethod

class LedgerBase(ABC):
    """Base class for all ledgers."""

    def _validate_ledger(self, ledger, new_state):
        if not isinstance(new_state, ledger):
            raise ValueError(
                f"Invalid type {new_state}. New state must be an instance of {ledger}"
            )

    @property
    def ledger(self):
        """
        Returns:
            (dict): Dictionary representing the ledger balance
        """
        return self._ledger

    @ledger.setter
    def ledger(self, new_state):
        """
        Args:
            new_state (dict): Dictionary representing the new state of the ledger balance

        Returns:
            (dict): Dictionary representing the updated ledger balance
        """
        self._validate_balance(type(self.ledger), new_state)
        self._ledger = new_state


from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

extras_require = {}
extras_require['complete'] = sorted(set(sum(extras_require.values(), [])))

setup(
    name='bancorml',
    version='0.1.7',
    author='Bancor Network',
    author_email='mike@bancor.network',
    description='BancorML is a library that builds, optimizes, and evaluates machine learning pipelines in the context of a multi-agent system',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/gcode-ai/bancorml',
    install_requires=open('requirements.txt').readlines(),
    extras_require=extras_require,
    tests_require=open('test-requirements.txt').readlines(),
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
          'bancorml = bancorml.__main__:cli'
        ]
    },
    data_files=[
        ('bancorml/tests/data', [
                                'bancorml/tests/environment_tests/data/withdrawal_algorithm_tests.json'
                              ])
                ],
)
